from celery import task
from emailsys import default
from emailsys.models import Message

@task
def webservice_send_mail(subject, emails, content):
    """
    apply default.sender inside the task
    """

    msg = Message(subject)
    msg.emails = emails
    msg.content = content
    default.webservice.SendMessageToEmails(msg)

# @task
# def django_send_mail(subject, emails, content):
#     pass



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

## Default WebService
DEFAULT_WEBSERVICE = 'webservice.activetrail.Activetrail'

## Default Sender (django, activetrail)
## django - the regular send_mail function send_mail (https://docs.djangoproject.com/en/dev/topics/email/)
DEFAULT_SENDER = 'emailsys.senders.ActivetrailSender'

# Default content type (HTML, TXT)
DEFAULT_CONTENT_TYPE = 'HTML'

# Default lang id (119 = Hebrew)
DEFAULT_LANGID = 119

USE_CELERY = getattr(settings, "EMAIL_SYS_USE_CELERY", True)
SENDER = getattr(settings, "EMAIL_SYS_SENDER", DEFAULT_SENDER)
WEBSERVICE = getattr(settings, "EMAIL_SYS_WEBSERVICE", DEFAULT_WEBSERVICE)
CONTENT_TYPE = getattr(settings, "EMAIL_SYS_CONTET_TYPE", DEFAULT_CONTENT_TYPE)
FROM_NAME = getattr(settings, "EMAIL_SYS_FROM_NAME", None)
FROM_ADDRESS = getattr(settings, "EMAIL_SYS_FROM_ADDRESS", None)
REPLAY_TO = getattr(settings, "EMAIL_SYS_REPLAY_TO", FROM_ADDRESS)

ACTIVETRAIL_USER = getattr(settings, "EMAIL_SYS_ACTIVETRAIL_USER", None)
ACTIVETRAIL_PASS = getattr(settings, "EMAIL_SYS_ACTIVETRAIL_PASS", None)
ACTIVETRAIL_LANGID = getattr(settings, "EMAIL_SYS_ACTIVETRAIL_LANGID", DEFAULT_LANGID)

if USE_CELERY:
    try:
        import celery
    except ImportError:
        raise ImproperlyConfigured("Could not import celery")



from emailsys.settings import FROM_ADDRESS, FROM_NAME, REPLAY_TO, CONTENT_TYPE
from emailsys.helpers import get_message_template 

class Message(object):
    # settings
    from_addr = FROM_ADDRESS
    from_name = FROM_NAME
    reply_to = REPLAY_TO
    encoding = 'utf-8'
    content_type = CONTENT_TYPE
    
    subject = None
    content = None
    emails = None
    template = None
    
    def __init__(self, subject, emails=None, content=None):
        self.subject = subject
        self.content = content
        self.emails = emails

    def __unicode__(self):
        return self.emails

    def add_email(self, _email):
        if self.emails is None:
            self.emails = []
            
        self.emails.append(_email)
    
    def use_template(self, template, context):
        # need to know what to pass with context
        self.content = get_message_template(template, context)
        pass


from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module
from django.template import loader, Context
from emailsys.settings import CONTENT_TYPE

def get_module_class(class_path):
    """
    imports and returns module class from ``path.to.module.Class`` argument
    """
    mod_name, cls_name = class_path.rsplit('.', 1)

    try:
        mod = import_module(mod_name)
    except ImportError as e:
        raise ImproperlyConfigured(('Error importing module %s: "%s"' % (mod_name, e)))

    return getattr(mod, cls_name)

def get_message_template(template, context=None):
    """
    Send email rendering text and html versions for the specified
    template name using the context dictionary passed in.
    """
    if context is None:
        context = {}

    # Loads a template passing in vars as context.
    def render(ext):
        name = "emailsys/%s.%s" % (template, ext)
        return loader.get_template(name).render(Context(context))

    return render(CONTENT_TYPE.lower())


'''
Created on Dec 3, 2013

@author: Ron Sneh <me@ronsneh.com>
'''

from emailsys import settings
from suds.client import Client
from suds.xsd.doctor import ImportDoctor, Import 

class Activetrail(object):
    url = 'http://webapi.mymarketing.co.il/Messaging/MessagingService.asmx?WSDL'
    client = None
    session_id = None
    
    def __init__(self):
        self._login()

    def _connect(self):
        imp = Import('http://www.w3.org/2001/XMLSchema') # the schema to import.
        imp.filter.add('http://tempuri.org/') # the schema to import into.
        d = ImportDoctor(imp)
        
        self.client = Client(self.url, doctor=d)

    def _login(self):
        self._connect()
        self._authHeader()
        
        login_result = self.client.service.Login()
        
        if login_result is None:
            # Maybe we need to connect again ?
            raise Exception('Do something?')
        
        self.session_id = login_result
        
    def _authHeader(self):
        auth_header = self.client.factory.create('AuthHeader')
        auth_header.Username = settings.ACTIVETRAIL_USER
        auth_header.Password = settings.ACTIVETRAIL_PASS
        auth_header.Token = self.session_id
        
        self.client.set_options(soapheaders=auth_header)
        
    def SendMessageToEmails(self, Message):
        self._connect()
        self._authHeader()

        # from address for webMessage
        address = self.client.factory.create('Address')
        # Should be taken from settings
        address.FromName = Message.from_name
        address.FromEmail = Message.from_addr
        
        
        webMessage = self.client.factory.create('WebMessage')
        webMessage.Subject = Message.subject
        webMessage.From = address
        webMessage.ExternalMessageId = '15' # Unknown
        webMessage.LanguageId = settings.ACTIVETRAIL_LANGID;
        webMessage.UserPlaceholders = False;
        webMessage.AddStatistics = False;
        webMessage.AddAdvertisement = False;
        webMessage.SignMessage = False;
        webMessage.AddUnsubscribeLink = False;
        webMessage.AddPrintButton = False;
        
        # This name is the campign name
        # webMessage.Name = "";

        # Need to know what this priority means
        webMessage.Priority = '1';
        
        arrayOfBodyParts = self.client.factory.create('ArrayOfBodyPart')
        bodyParts = self.client.factory.create('BodyPart')
#         bodyPartFormat = self.client.factory.create('BodyPartFormat')

        bodyParts.BodyPartEncoding = Message.encoding
        bodyParts.BodyPartFormat = Message.content_type
        bodyParts.Body = Message.content

        arrayOfBodyParts.BodyPart = bodyParts        
        webMessage.BodyParts = arrayOfBodyParts

        emails = self.client.factory.create('ArrayOfString')

        for email in Message.emails:
            emails.string.append(email)

        try:        
            # We should get the number of the total sent amount
            response = self.client.service.SendMessageToEmails(webMessage, emails)
        except IOError as e:
            print "I/O error({0}): {1}".format(e.errno, e.strerror)
